@@ -12,4 +12,4 @@ PROGUARD_HOME=`dirname "$0"`/..
 # On Linux, Java 1.6.0_24 and higher hang when starting the GUI:
 #   http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=7027598
 # We're using the -D option as a workaround.
-java -DsuppressSwingDropSupport=true -jar $PROGUARD_HOME/lib/proguardgui.jar "$@"
+java -DsuppressSwingDropSupport=true -jar "$PROGUARD_HOME/lib/proguardgui.jar" "$@"
