@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -115,6 +115,7 @@ public class InputReader
         {
             System.err.println("Note: there were " + noteCount +
                                " duplicate class definitions.");
+            System.err.println("      (http://proguard.sourceforge.net/manual/troubleshooting.html#duplicateclass)");
         }
 
         // Print out a summary of the warnings, if necessary.
@@ -125,6 +126,7 @@ public class InputReader
                                " classes in incorrectly named files.");
             System.err.println("         You should make sure all file names correspond to their class names.");
             System.err.println("         The directory hierarchies must correspond to the package hierarchies.");
+            System.err.println("         (http://proguard.sourceforge.net/manual/troubleshooting.html#unexpectedclass)");
 
             if (!configuration.ignoreWarnings)
             {
