@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -137,7 +137,7 @@ implements   AttributeVisitor,
             Instruction replacementInstruction =
                 new VariableInstruction(variableInstruction.opcode,
                                         newVariableIndex,
-                                        variableInstruction.constant).shrink();
+                                        variableInstruction.constant);
 
             codeAttributeEditor.replaceInstruction(offset, replacementInstruction);
         }
