@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -174,7 +174,6 @@ public class BranchInstruction extends Instruction
      */
     private int requiredBranchOffsetSize()
     {
-        return branchOffset << 16 >> 16 == branchOffset ? 2 :
-                                                          4;
+        return (short)branchOffset == branchOffset ? 2 : 4;
     }
 }
