@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -47,7 +47,8 @@ public class MemberNameFilter implements MemberVisitor
     public MemberNameFilter(String        regularExpression,
                             MemberVisitor memberVisitor)
     {
-        this(new NameParser().parse(regularExpression), memberVisitor);
+        this(new ListParser(new NameParser()).parse(regularExpression),
+             memberVisitor);
     }
 
 
