@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -20,12 +20,6 @@
  */
 package proguard.io;
 
-import proguard.classfile.ClassConstants;
-import proguard.util.ExtensionMatcher;
-
-import java.io.IOException;
-
-
 /**
  * This DataEntryReader delegates to one of two other DataEntryReader instances,
  * depending on whether the data entry represents a directory or not.
