@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -164,6 +164,14 @@ implements   ClassVisitor,
     }
 
 
+    public void visitLibraryClass(LibraryClass libraryClass)
+    {
+        // This can happen for dubious input, if the outer class of a program
+        // class is a library class, and its name is requested.
+        newClassName = libraryClass.getName();
+    }
+
+
     // Implementations for AttributeVisitor.
 
     public void visitAnyAttribute(Clazz clazz, Attribute attribute) {}
