@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2013 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -30,6 +30,7 @@ import proguard.classfile.instruction.*;
 import proguard.classfile.instruction.visitor.InstructionVisitor;
 import proguard.classfile.util.*;
 import proguard.classfile.visitor.*;
+import proguard.optimize.KeepMarker;
 import proguard.optimize.info.*;
 
 import java.util.*;
@@ -154,6 +155,8 @@ implements   AttributeVisitor,
                 System.err.println("  Inlined method = ["+method.getName(clazz)+method.getDescriptor(clazz)+"]");
             }
             System.err.println("  Exception      = ["+ex.getClass().getName()+"] ("+ex.getMessage()+")");
+
+            ex.printStackTrace();
             System.err.println("Not inlining this method");
 
             if (DEBUG)
@@ -312,7 +315,7 @@ implements   AttributeVisitor,
                 }
 
                 codeAttributeComposer.appendInstruction(parameterSize-parameterIndex-1,
-                                                        new VariableInstruction(opcode, variableOffset + parameterOffset + parameterIndex).shrink());
+                                                        new VariableInstruction(opcode, variableOffset + parameterOffset + parameterIndex));
             }
         }
 
@@ -320,7 +323,7 @@ implements   AttributeVisitor,
         if (!isStatic)
         {
             codeAttributeComposer.appendInstruction(parameterSize,
-                                                    new VariableInstruction(InstructionConstants.OP_ASTORE, variableOffset).shrink());
+                                                    new VariableInstruction(InstructionConstants.OP_ASTORE, variableOffset));
         }
 
         codeAttributeComposer.endCodeFragment();
@@ -353,7 +356,7 @@ implements   AttributeVisitor,
 
     public void visitAnyInstruction(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, Instruction instruction)
     {
-        codeAttributeComposer.appendInstruction(offset, instruction.shrink());
+        codeAttributeComposer.appendInstruction(offset, instruction);
     }
 
 
@@ -380,7 +383,7 @@ implements   AttributeVisitor,
                                                   codeAttribute.u4codeLength - offset);
 
                         codeAttributeComposer.appendInstruction(offset,
-                                                                branchInstruction.shrink());
+                                                                branchInstruction);
                     }
                     else
                     {
@@ -393,7 +396,7 @@ implements   AttributeVisitor,
             }
         }
 
-        codeAttributeComposer.appendInstruction(offset, simpleInstruction.shrink());
+        codeAttributeComposer.appendInstruction(offset, simpleInstruction);
     }
 
 
@@ -406,7 +409,7 @@ implements   AttributeVisitor,
             variableInstruction.variableIndex += variableOffset;
         }
 
-        codeAttributeComposer.appendInstruction(offset, variableInstruction.shrink());
+        codeAttributeComposer.appendInstruction(offset, variableInstruction);
     }
 
 
@@ -464,7 +467,7 @@ implements   AttributeVisitor,
                 constantAdder.addConstant(clazz, constantInstruction.constantIndex);
         }
 
-        codeAttributeComposer.appendInstruction(offset, constantInstruction.shrink());
+        codeAttributeComposer.appendInstruction(offset, constantInstruction);
     }
 
 
@@ -488,7 +491,10 @@ implements   AttributeVisitor,
     {
         int accessFlags = programMethod.getAccessFlags();
 
-        if (// Only inline the method if it is private, static, or final.
+        if (// Don't inline methods that must be preserved.
+            !KeepMarker.isKept(programMethod)                                                     &&
+
+            // Only inline the method if it is private, static, or final.
             (accessFlags & (ClassConstants.INTERNAL_ACC_PRIVATE |
                             ClassConstants.INTERNAL_ACC_STATIC  |
                             ClassConstants.INTERNAL_ACC_FINAL)) != 0                              &&
@@ -550,9 +556,11 @@ implements   AttributeVisitor,
 
             // Only inline the method if it comes from the a class with at most
             // a subset of the initialized superclasses.
-            (programClass.equals(targetClass) ||
+            ((accessFlags & ClassConstants.INTERNAL_ACC_STATIC) == 0 ||
+             programClass.equals(targetClass)                        ||
              initializedSuperClasses(targetClass).containsAll(initializedSuperClasses(programClass))))
-        {                                                                                                   boolean oldInlining = inlining;
+        {
+            boolean oldInlining = inlining;
             inlining = true;
             inliningMethods.push(programMethod);
 
